@@ -11,6 +11,7 @@
 
 import logging
 import numpy as np
+import time
 import warnings
 import parakeet.config
 import parakeet.dqe
@@ -302,8 +303,29 @@ class ExitWaveImageSimulator(object):
         psi_tot = np.abs(image) ** 2
         logger.info("Ideal image min/max: %f/%f" % (np.min(psi_tot), np.max(psi_tot)))
 
+        # Get the timestamp
+        timestamp = time.time_ns() / 1e9
+
+        # Set the metaadata
+        metadata = self.metadata[index]
+        metadata["timestamp"] = timestamp
+        metadata["tilt_alpha"] = angle
+        metadata["stage_z"] = position[2]
+        metadata["shift_x"] = position[0]
+        metadata["shift_y"] = position[1]
+        metadata["shift_offset_x"] = driftx
+        metadata["shift_offset_y"] = drifty
+        metadata["energy"] = self.microscope.beam.energy
+        metadata["theta"] = self.microscope.beam.theta
+        metadata["phi"] = self.microscope.beam.phi
+        metadata["image_size_x"] = nx
+        metadata["image_size_y"] = ny
+        metadata["ice"] = self.simulation["ice"]
+        metadata["damage_model"] = self.simulation["radiation_damage_model"]
+        metadata["sensitivity_coefficient"] = self.simulation["sensitivity_coefficient"]
+
         # Compute the image scaled with Poisson noise
-        return (index, angle, position, image, (driftx, drifty), None)
+        return (index, image, metadata)
 
 
 def simulation_factory(
@@ -438,4 +460,5 @@ def _exit_wave_Config(
     )
 
     # Run the simulation
+    simulation.simulate_image.metadata = writer.header
     simulation.run(writer)
